@@ -3,11 +3,13 @@
 
     nereid_wishlist test suite
 
-    :copyright: (c) 2014 by Openlabs Technologies & Consulting (P) Ltd.
+    :copyright: (c) 2014-2015 by Openlabs Technologies & Consulting (P) Ltd.
     :license: GPLv3, see LICENSE for more details
 '''
 import unittest
 from decimal import Decimal
+from dateutil.relativedelta import relativedelta
+import datetime
 
 import pycountry
 import trytond.tests.test_tryton
@@ -37,6 +39,8 @@ class TestWishlist(NereidTestCase):
         self.Uom = POOL.get('product.uom')
         self.Template = POOL.get('product.template')
         self.Location = POOL.get('stock.location')
+        self.PriceList = POOL.get('product.price_list')
+        self.SaleShop = POOL.get('sale.shop')
 
         self.templates = {
             'wishlists.jinja':
@@ -45,6 +49,32 @@ class TestWishlist(NereidTestCase):
                 '{{ wishlist.name }}',
         }
 
+    def _create_payment_term(self):
+        """Create a simple payment term with all advance
+        """
+        PaymentTerm = POOL.get('account.invoice.payment_term')
+
+        return PaymentTerm.create([{
+            'name': 'Direct',
+            'lines': [('create', [{'type': 'remainder'}])]
+        }])
+
+    def _create_pricelists(self):
+        """
+        Create the pricelists
+        """
+        # Setup the pricelists
+        self.guest_pl_margin = Decimal('1.20')
+        return self.PriceList.create([{
+            'name': 'PL 2',
+            'company': self.company.id,
+            'lines': [
+                ('create', [{
+                    'formula': 'unit_price * %s' % self.guest_pl_margin
+                }])
+            ],
+        }])[0]
+
     def _create_countries(self, count=5):
         """
         Create some sample countries and subdivisions
@@ -69,6 +99,95 @@ class TestWishlist(NereidTestCase):
                         'type': subdivision.type.lower(),
                     }])
 
+    def _create_coa_minimal(self, company):
+        """Create a minimal chart of accounts
+        """
+        AccountTemplate = POOL.get('account.account.template')
+        Account = POOL.get('account.account')
+
+        account_create_chart = POOL.get(
+            'account.create_chart', type="wizard")
+
+        account_template, = AccountTemplate.search(
+            [('parent', '=', None)]
+        )
+
+        session_id, _, _ = account_create_chart.create()
+        create_chart = account_create_chart(session_id)
+        create_chart.account.account_template = account_template
+        create_chart.account.company = company
+        create_chart.transition_create_account()
+
+        receivable, = Account.search([
+            ('kind', '=', 'receivable'),
+            ('company', '=', company),
+        ])
+        payable, = Account.search([
+            ('kind', '=', 'payable'),
+            ('company', '=', company),
+        ])
+        create_chart.properties.company = company
+        create_chart.properties.account_receivable = receivable
+        create_chart.properties.account_payable = payable
+        create_chart.transition_create_properties()
+
+    def _get_account_by_kind(self, kind, company=None, silent=True):
+        """Returns an account with given spec
+        :param kind: receivable/payable/expense/revenue
+        :param silent: dont raise error if account is not found
+        """
+        Account = POOL.get('account.account')
+        Company = POOL.get('company.company')
+
+        if company is None:
+            company, = Company.search([], limit=1)
+
+        accounts = Account.search([
+            ('kind', '=', kind),
+            ('company', '=', company)
+        ], limit=1)
+        if not accounts and not silent:
+            raise Exception("Account not found")
+        return accounts[0] if accounts else None
+
+    def _create_fiscal_year(self, date=None, company=None):
+        """
+        Creates a fiscal year and requried sequences
+        """
+        FiscalYear = POOL.get('account.fiscalyear')
+        Sequence = POOL.get('ir.sequence')
+        SequenceStrict = POOL.get('ir.sequence.strict')
+        Company = POOL.get('company.company')
+
+        if date is None:
+            date = datetime.date.today()
+
+        if company is None:
+            company, = Company.search([], limit=1)
+
+        invoice_sequence, = SequenceStrict.create([{
+            'name': '%s' % date.year,
+            'code': 'account.invoice',
+            'company': company,
+        }])
+        fiscal_year, = FiscalYear.create([{
+            'name': '%s' % date.year,
+            'start_date': date + relativedelta(month=1, day=1),
+            'end_date': date + relativedelta(month=12, day=31),
+            'company': company,
+            'post_move_sequence': Sequence.create([{
+                'name': '%s' % date.year,
+                'code': 'account.move',
+                'company': company,
+            }])[0],
+            'out_invoice_sequence': invoice_sequence,
+            'in_invoice_sequence': invoice_sequence,
+            'out_credit_note_sequence': invoice_sequence,
+            'in_credit_note_sequence': invoice_sequence,
+        }])
+        FiscalYear.create_period([fiscal_year])
+        return fiscal_year
+
     def login(self, client, username, password, assert_=True):
         """
         Tries to login.
@@ -105,8 +224,31 @@ class TestWishlist(NereidTestCase):
         }])
 
         # Create parties
-        self.party, = self.Party.create([{
-            'name': 'openlabs',
+        with Transaction().set_context(company=None):
+            self.party, = self.Party.create([{
+                'name': 'openlabs',
+            }])
+
+        self.company, = self.Company.create([{
+            'party': self.party,
+            'currency': usd,
+        }])
+
+        self.User.write(
+            [self.User(USER)], {
+                'main_company': self.company.id,
+                'company': self.company.id,
+            }
+        )
+        CONTEXT.update(self.User.get_preferences(context_only=True))
+
+        # Create Fiscal Year
+        self._create_fiscal_year(company=self.company.id)
+        # Create Chart of Accounts
+        self._create_coa_minimal(company=self.company.id)
+
+        guest_party, = self.Party.create([{
+            'name': 'Guest User',
         }])
 
         self.party2, = self.Party.create([{
@@ -117,9 +259,12 @@ class TestWishlist(NereidTestCase):
             'name': 'Registered User 2',
         }])
 
-        self.company, = self.Company.create([{
-            'party': self.party,
-            'currency': usd,
+        self.guest_user, = self.NereidUser.create([{
+            'party': guest_party.id,
+            'display_name': 'Guest User',
+            'email': 'guest@openlabs.co.in',
+            'password': 'password',
+            'company': self.company.id,
         }])
 
         # Create test users
@@ -155,14 +300,32 @@ class TestWishlist(NereidTestCase):
         location, = self.Location.search([
             ('type', '=', 'storage')
         ], limit=1)
+
+        payment_term, = self._create_payment_term()
+
+        shop_price_list = self._create_pricelists()
+
+        with Transaction().set_context(company=self.company.id):
+            self.shop, = self.SaleShop.create([{
+                'name': 'Default Shop',
+                'price_list': shop_price_list,
+                'warehouse': warehouse,
+                'payment_term': payment_term,
+                'company': self.company.id,
+                'users': [('add', [USER])]
+            }])
+
+        self.User.set_preferences({'shop': self.shop})
         self.NereidWebsite.create([{
             'name': 'localhost',
             'company': self.company.id,
             'application_user': USER,
+            'shop': self.shop,
             'default_locale': self.locale_en_us.id,
             'warehouse': warehouse,
             'stock_location': location,
             'countries': [('add', self.available_countries)],
+            'guest_user': self.guest_user,
         }])
 
     def test_0010_create_wishlist(self):
@@ -171,7 +334,7 @@ class TestWishlist(NereidTestCase):
         If wishlist already exist just return that wishlist.
         """
 
-        with Transaction().start(DB_NAME, USER, CONTEXT):
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
             self.setup_defaults()
             app = self.get_app()
 
@@ -223,7 +386,7 @@ class TestWishlist(NereidTestCase):
         Test to view all wishlist
         """
 
-        with Transaction().start(DB_NAME, USER, CONTEXT):
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
             self.setup_defaults()
             app = self.get_app()
 
@@ -252,7 +415,7 @@ class TestWishlist(NereidTestCase):
         Test to remove wishlist
         """
 
-        with Transaction().start(DB_NAME, USER, CONTEXT):
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
             self.setup_defaults()
             app = self.get_app()
 
@@ -299,7 +462,7 @@ class TestWishlist(NereidTestCase):
         Test to add/remove a product to wishlist.
         """
 
-        with Transaction().start(DB_NAME, USER, CONTEXT):
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
             self.setup_defaults()
             app = self.get_app()
 
@@ -433,7 +596,7 @@ class TestWishlist(NereidTestCase):
         Test to render single wishlist.
         """
 
-        with Transaction().start(DB_NAME, USER, CONTEXT):
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
             self.setup_defaults()
             app = self.get_app()
 
@@ -481,7 +644,7 @@ class TestWishlist(NereidTestCase):
 
         Wishlist = POOL.get('wishlist.wishlist')
 
-        with Transaction().start(DB_NAME, USER, CONTEXT):
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
             self.setup_defaults()
             app = self.get_app()
 
